// StealthList.cpp: implementation of the CStealthList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "StealthList.h"
#include "CallList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStealthList::CStealthList()
{
	m_pStealthListINT  = new CMapStealthULongToPtr;
	m_pStealthListTEXT = new CMapStealthStrToPtr;

	m_lStealthIdCount = 0;
}

CStealthList::~CStealthList()
{
	DeleteAll();
	
	delete m_pStealthListTEXT;
	delete m_pStealthListINT;
}

ULONG CStealthList::ChooseStealthId()
{
	while(TRUE)
	{
		m_lStealthIdCount++;
		
		if(FindStealthCall(m_lStealthIdCount) == NULL) break;
	}

	return m_lStealthIdCount;
}

void CStealthList::AddCallToStealthList(CCallData* pCallData)
{
	pCallData->m_lStealthId = ChooseStealthId();
	 	
	m_pStealthListINT->SetAt(pCallData->m_lStealthId, pCallData);
	m_pStealthListTEXT->SetAt(pCallData->m_sStealthDialNo, pCallData);
}

void CStealthList::FreeCaptureCall(CCallData* pCallData)
{
	if(pCallData->m_lStealthId != 0)
	{
		CCallData* pStealthCallData = FindStealthCall(pCallData->m_lStealthId);
		
		if(pStealthCallData != NULL)
		{
			pStealthCallData->m_bStealthListening = FALSE;
			
			long lStealthSessionId = CServerSIP::m_pServerSIP->m_objServerData.GetStealthSessionId();
			CServerSIP::m_pServerSIP->AddCallToSession(pStealthCallData, lStealthSessionId);
		}
	}
}

void CStealthList::TryCaptureCall(CCallData* pCallData)
{
	CCallData* pStealthCallData = FindStealthCall(pCallData->m_sUserId);
	if(pStealthCallData == NULL || pStealthCallData->m_bStealthListening == TRUE) return;

	CServerSIP::m_pServerSIP->AddCallToSession(pStealthCallData, pCallData->m_lSessionId);
	
	pStealthCallData->m_bStealthListening = TRUE;
	pCallData->m_lStealthId = pStealthCallData->m_lStealthId;
}


CCallData* CStealthList::FindStealthCall(ULONG lStealthId)
{
	CCallData* pCallData;
	
	if(!m_pStealthListINT->Lookup(lStealthId, pCallData))
		return NULL;
	
	return pCallData;
}

CCallData* CStealthList::FindStealthCall(CString sDialNo)
{
	sDialNo.MakeLower();
	
	CCallData* pCallData;
	
	if(!m_pStealthListTEXT->Lookup(sDialNo, pCallData))
		return NULL;
	
	return pCallData;
}

void CStealthList::RemoveStealthCall(CCallData* pCallData)
{
	if(!m_pStealthListINT->RemoveKey(pCallData->m_lStealthId))
		return;

	m_pStealthListTEXT->RemoveKey(pCallData->m_sStealthDialNo);
}

void CStealthList::DeleteAll()
{
	ULONG lKey;
	CCallData* pCallData;

	for(POSITION pos = m_pStealthListINT->GetStartPosition(); pos != NULL;)
	{
		m_pStealthListINT->GetNextAssoc( pos, lKey, pCallData);
		delete pCallData;
	}

	m_pStealthListTEXT->RemoveAll();
}